/*********************************************************************************
 * libcapifax.h
 *
 * Copyright (c) 2008-2009 Marco Zissen <maz@v3v.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *********************************************************************************/
#ifdef WIN32
#include <windows.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <fcntl.h>
#include <signal.h>

#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#endif

#ifndef WIN32
#include <pthread.h>
#include <sched.h>
#include <sys/resource.h>
#endif

// CAPI 
#include <capi20.h>
#ifndef WIN32
#include <linux/capi.h>
#define STDCALL 
#else
#include <cwin32.h>
#define usleep Sleep
#define snprintf _snprintf
#define STDCALL __stdcall

#define ALERT_REQ(a,b,c,d,e,f,g,h,i) ALERT_REQ(a,b,c,d,e,f,g,h)
#define CONNECT_REQ(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x) CONNECT_REQ(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v)
#define CONNECT_RESP(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s) CONNECT_RESP(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r)
#define CAPI20_WaitforMessage(a,b) CAPI20_WAIT_FOR_SIGNAL(a)
#define capi_get_cmsg(a,b) CAPI_GET_CMSG(a,b)

typedef struct capi_profile
{
  WORD ncontroller;
  WORD nbchannel;
  DWORD goptions;
  DWORD support1;
  DWORD support2;
  DWORD support3;
  BYTE reserved[24];
  BYTE manu[20];
} capi_profile;

#ifndef _TM_DEFINED
struct tm {
        int tm_sec;     /* seconds after the minute - [0,59] */
        int tm_min;     /* minutes after the hour - [0,59] */
        int tm_hour;    /* hours since midnight - [0,23] */
        int tm_mday;    /* day of the month - [1,31] */
        int tm_mon;     /* months since January - [0,11] */
        int tm_year;    /* years since 1900 */
        int tm_wday;    /* days since Sunday - [0,6] */
        int tm_yday;    /* days since January 1 - [0,365] */
        int tm_isdst;   /* daylight savings time flag */
        };
#define _TM_DEFINED
#endif
#endif

#include <spandsp.h>

#if (SPANDSP_VERSION < 4 || SPANDSP_VERSION > 6)
#undef SPANDSP_VERSION
#define SPANDSP_VERSION         6       // 6, 5 or 4 (currently tested with spandsp v0.6_pre7)
#endif
#if (CAPI_CONNECTIONS < 1 || CAPI_CONNECTIONS > 10)
#undef CAPI_CONNECTIONS
#define CAPI_CONNECTIONS        5       // max. CAPI connections
#endif
#if (CAPI_PACKETS < 160 || CAPI_PACKETS > 1024)
#undef CAPI_PACKETS
#define CAPI_PACKETS            160     // Packet size
#endif
#if (CAPI_BUFFERCNT < 1 || CAPI_BUFFERCNT > 10)
#undef CAPI_BUFFERCNT
#define CAPI_BUFFERCNT          6       // Packer buffer count
#endif
#if (CAPI_BCHANNELS < 1 || CAPI_BCHANNELS > 10)
#undef CAPI_BCHANNELS
#define CAPI_BCHANNELS          2       // max. B-Channels
#endif
#if (CAPI_ULAW <= 0 || CAPI_ULAW > 1)
#undef CAPI_ULAW
#define CAPI_ULAW               0       // Alaw=0, Ulaw=1
#endif
#if (CAPI_CIP != 0x11 && CAPI_CIP != 0x04)
#undef CAPI_CIP
#define CAPI_CIP                0x11    // Service indicator (0x04=speech, 0x11=fax/g3)
#endif
#if (WITH_BOOST <=0 || WITH_BOOST > 1)
#undef WITH_BOOST
#define WITH_BOOST              0x01    // Enable/disable process boost
#endif

#define EXTERN                  extern
#define LIBNAME                 "libcapifax"
#define LIBVER                  "v0.7.3-jmb4"
#define LIBCR                   "(c) 2008-2009 Marco Zissen"

typedef enum capifax_phase_t
{
    IDLE         =   -1,
    CONNECT      =   1,
    PHASE_B      =   2,
    PHASE_D      =   3,
    PHASE_E      =   4,
    FAX_OK       =   5,
    FAX_FAILED   =   6,
    FAX_REJECTED =   7
} 
capifax_phase_t;

typedef struct capifax_status_t
{
    char                tiff_file[256];

    char                src_no[64];
    char                trg_no[64];
    char                ident[64];
    char                header[64];
    
    capifax_phase_t     phase;
    int                 error_code;
    int                 capi_code;
    int                 sending;

    int                 ecm;
    int                 modem;
    int                 bitrate;
    int                 encoding;
    int                 bad_rows;

    int                 page_current;
    int                 page_total;

    int                 bytes_received;
    int                 bytes_sent;
    int                 bytes_total;

    int                 manual_hookup;

} capifax_status_t;

typedef struct capifax_con_t
{
    fax_state_t         *fax;    
    capifax_status_t    status;    

    unsigned int        controller;
    unsigned long int   PLCI;
    unsigned long int   NCCI;    
    _cword              hdl;    
} 
capifax_con_t;

typedef void (STDCALL *capifax_status)(int con, capifax_status_t *status);
typedef void (STDCALL *capifax_progress)(int con, capifax_status_t *status);
typedef void (STDCALL *capifax_log)(char *log);
typedef int  (STDCALL *capifax_receive)(int con, char *sender, char *target, char **file);

EXTERN int capifax_init(int controller, int loglevel, capifax_status status_event, capifax_progress progress_event, capifax_log log_event, capifax_receive receive_event);
EXTERN int capifax_close();
EXTERN int capifax_loop();
EXTERN int capifax_send(const char *tiff_file, int modem, int ecm, int controller, const char *src_no, const char *trg_no, const char *LSI, const char *LocalHeaderInfo, int call_anonymous);
EXTERN int capifax_recv(const char *tiff_file, int modem, int ecm, const char *src_no, char *trg_no, const char *LSI, const char *LocalHeaderInfo, int manual_hookup);
EXTERN int capifax_terminate(int con);
EXTERN int capifax_accept(int con);
EXTERN int capifax_reject(int con);



